# -*- coding: utf-8 -*-
from django.test import TestCase
from django.utils import timezone
from yamoney.models import Transaction


class TransactionTestCase(TestCase):

    def test_transaction_create(self):
        """Transaction must be created"""
        test_data = {
            'notification_type': Transaction.NOTIFICATION_TYPE_CHOICES[0][0],
            'operation_id': '123',
            'amount': 100,
            'withdraw_amount': 100,
            'datetime': timezone.now(),
            'sender': '9999999999',
            'codepro': False
        }
        test_obj = Transaction.objects.create(**test_data)
        transaction = Transaction(**test_data)
        transaction.label = Transaction.generate_label(test_obj)
        transaction.save()
        self.assertEqual(transaction.get_related_obj(), test_obj)


# -*- coding: utf-8 -*-
from django.contrib import admin
from yamoney.models import Transaction


class TransactionAdmin(admin.ModelAdmin):
    list_display = ('notification_type', 'withdraw_amount', 'amount', 'currency', 'datetime',)

    def has_add_permission(self, request):
        return False


admin.site.register(Transaction, TransactionAdmin)


# -*- coding: utf-8 -*-
from hashlib import sha1
from django.db import models
from django import forms
from yamoney import settings as yamoney_settings
from yamoney.models import Transaction
from yamoney.fields import DateTimeISO6801Field


class YandexPaymentForm(forms.Form):
    ACTION_URL = 'https://money.yandex.ru/quickpay/confirm.xml'
    QUICKPAY_CHOICES = (
        ('shop', 'shop'),
        ('donate', 'donate'),
        ('small', 'small'),
    )
    PAYMENTTYPE_CHOICES = (
        ('PC', u'оплата со счета Яндекс.Денег'),
        ('AC', u'оплата с банковской карты')
    )
    FIELD_NAME_MAPPING = {
        'shortDest': 'short-dest',
        'quickpayForm': 'quickpay-form',
    }
    receiver = forms.CharField(widget=forms.HiddenInput)
    formcomment = forms.CharField(max_length=50, widget=forms.HiddenInput)
    shortDest = forms.CharField(max_length=50, widget=forms.HiddenInput)
    quickpayForm = forms.ChoiceField(choices=QUICKPAY_CHOICES, initial='shop', widget=forms.HiddenInput)
    targets = forms.CharField(max_length=150, widget=forms.HiddenInput)
    sum = forms.FloatField(widget=forms.HiddenInput)
    paymentType = forms.ChoiceField(label=u'Варианты оплаты', choices=PAYMENTTYPE_CHOICES,
                                    initial='AC',
                                    widget=forms.RadioSelect)
    label = forms.CharField(required=False, widget=forms.HiddenInput)
    # comment = forms.CharField(required=False, max_length=200, widget=forms.HiddenInput)

    def add_prefix(self, field_name):
        field_name = self.FIELD_NAME_MAPPING.get(field_name, field_name)
        return super(YandexPaymentForm, self).add_prefix(field_name)


class YandexNotificationForm(forms.ModelForm):
    datetime = DateTimeISO6801Field()
    sha1_hash = forms.CharField()

    class Meta:
        model = Transaction
        fields = '__all__'

    def make_hash(self):
        cd = self.data
        return sha1('&'.join(map(str, (
            cd.get('notification_type', ''),
            cd.get('operation_id', ''),
            cd.get('amount', ''),
            cd.get('currency', ''),
            cd.get('datetime', ''),
            cd.get('sender', ''),
            cd.get('codepro', ''),
            yamoney_settings.NOTIFICATION_SECRET,
            cd.get('label', ''),
        )))).hexdigest()
    
    def clean(self):
        cd = super(YandexNotificationForm, self).clean()
        sha1_hash = cd.get('sha1_hash')
        if sha1_hash != self.make_hash():
            raise forms.ValidationError(u'Хэш не совпадает')
        return cd


def paymentform_factory(targets, sum, label):
    if isinstance(label, models.Model):
        label = Transaction.generate_label(label)
    initial = {
        'receiver': yamoney_settings.ACCOUNT,
        'formcomment': yamoney_settings.FORM_COMMENT or targets,
        'shortDest': targets,
        'targets': targets,
        'sum': sum,
        'label': label,
    }
    return YandexPaymentForm(initial=initial)


# -*- coding: utf-8 -*-
import dateutil.parser
from django import forms


class DateTimeISO6801Field(forms.DateTimeField):

    def strptime(self, value, format):
        return dateutil.parser.parse(value)


import os
from setuptools import setup, find_packages
import yamoney

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-yamoney',
    version=yamoney.__version__,
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    description='A simple Django app for Yandex.Money service',
    long_description=README,
    author='Mpower',
    author_email='mpower.public@yandex.ru',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'Django>=1.7,<1.9',
        'python-dateutil>=2.4'
    ]
)


# -*- coding: utf-8 -*-
from django.conf import settings as django_settings


def get_setting(name, default):
    """
    A little helper for fetching global settings with a common prefix.
    """
    parent_name = "YAMONEY_{0}".format(name)
    return getattr(django_settings, parent_name, default)

ACCOUNT = get_setting('ACCOUNT', None)
FORM_COMMENT = get_setting('FORM_COMMENT', None)
NOTIFICATION_SECRET = get_setting('NOTIFICATION_SECRET', '')
MAIL_ADMINS_ON_TRANSACTION_ERROR = get_setting('MAIL_ADMINS_ON_TRANSACTION_ERROR', False)